# -*- coding: utf-8 -*-
from django.http import HttpResponse
from rest_framework.decorators import action
from rest_framework.permissions import IsAuthenticated, IsAdminUser, BasePermission
from rest_framework.response import Response
from rest_framework.viewsets import GenericViewSet
from rest_framework import status, mixins
from djangorestfilemanager.models import File
from djangorestfilemanager.filters import FileFilter
from djangorestfilemanager.serializers import FileSerializer, FileViewSerializer
import logging

logger = logging.getLogger(__name__)


# Create your views here.


class FileModelViewSet(mixins.CreateModelMixin, mixins.RetrieveModelMixin, GenericViewSet):
    queryset = File.objects.all()
    serializer_class = FileSerializer
    filter_class = FileFilter
    search_fields = ['name', 'uuid']
    ordering_fields = ['name', 'uuid', 'user_name', 'origin', 'type']
    permission_classes = (IsAuthenticated,)

    def get_serializer_class(self):
        if self.action == 'retrieve':
            return FileViewSerializer
        return FileSerializer

    def create(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        self.perform_create(serializer)
        headers = self.get_success_headers(serializer.data)
        instance = serializer.instance
        serializer = FileViewSerializer(instance, context={'request': request})
        return Response(serializer.data, status=status.HTTP_201_CREATED, headers=headers)

    @action(detail=False, methods=['get'], url_path='list-files', permission_classes=[IsAdminUser, IsAuthenticated])
    def list_files(self, request, *args, **kwargs):
        queryset = self.filter_queryset(self.get_queryset())
        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)

    @action(detail=True, methods=['post'])
    def file(self, request, pk, *args, **kwargs):
        object = self.get_object()
        if not request.user.is_superuser:
            if not object.share:
                if object.user_name != request.user.username:
                    return Response(status=status.HTTP_401_UNAUTHORIZED)
            else:

                if object.permission and not request.user.has_perm(object.permission):
                    return Response(status=status.HTTP_401_UNAUTHORIZED)
        filename = object.file.name.split('/')[-1]
        response = HttpResponse(object.file, content_type='text/plain')
        response['Content-Disposition'] = 'attachment; filename=%s' % filename
        logger.info('- File uuid: {}, download by user {}'.format(object.uuid, request.user.username))
        return response



# -*- coding: utf-8 -*-
from django_filters import rest_framework as filters
from djangorestfilemanager.models import File


class FileFilter(filters.FilterSet):
    class Meta:
        model = File
        fields = ['uuid', 'name', 'user_name', 'creation_date', 'last_mod_date', 'origin', 'type', 'share']



# Generated by Django 3.0 on 2019-12-10 08:06

from django.db import migrations, models
import djangorestfilemanager.models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='File',
            fields=[
                ('uuid', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False)),
                ('file', models.FileField(storage=djangorestfilemanager.models.MyFileSystemStorage(location='files'), upload_to='files/', verbose_name='File')),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('user_name', models.CharField(max_length=100, verbose_name='User name')),
                ('creation_date', models.DateTimeField(auto_now_add=True, verbose_name='Creation date')),
                ('last_mod_date', models.DateTimeField(auto_now=True, verbose_name='Creation date')),
                ('origin', models.CharField(blank=True, choices=[('RST', 'REST')], default='RST', max_length=3, null=True, verbose_name='Origin')),
                ('type', models.CharField(blank=True, choices=[('DFT', 'Default')], default='DFT', max_length=3, null=True, verbose_name='Type')),
                ('permission', models.CharField(blank=True, default='', max_length=200, null=True, verbose_name='Permission')),
                ('share', models.BooleanField(default=False, verbose_name='Share')),
            ],
            options={
                'verbose_name': 'File',
                'verbose_name_plural': 'Files',
            },
        ),
    ]



from django.contrib.auth.models import User
from django.test import override_settings
from rest_framework import status
from django.conf import settings
from rest_framework.test import APITestCase, APIClient

from djangorestfilemanager.models import File


@override_settings(ROOT_URLCONF='files.urls')
class TestUploadFile(APITestCase):
    def setUp(self):
        self.client = APIClient()

        self.user = User.objects.create(username='LUKE', password='1234qwer1234', email='luke@sw.stwars', is_staff=True,
                                        is_superuser=True)
        self.file = open('{}/files/test/files/test.png'.format(settings.BASE_DIR), 'rb')
        self.data = {
            'file': self.file,
            'name': 'test',
            'user_name': 'Luke SKW'
        }

    def test_upload_not_authenticated_file(self):
        file = {'file': (str(self.file.name), self.file, 'text/plain', {'Expires': '0'})}
        response = self.client.post('/files/', data=self.data, file=file)
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
        response = self.client.get('/files/list-files/')
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
        self.assertEqual(File.objects.all().count(), 0)

    def test_upload_file(self):
        self.client.force_authenticate(user=self.user)
        file = {'file': (str(self.file.name), self.file, 'text/plain', {'Expires': '0'})}
        response = self.client.post('/files/', data=self.data, file=file)
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)
        response = self.client.get('/files/list-files/')
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data.get('count'), 1)

    def test_list_not_superuser_file(self):
        self.test_upload_file()
        user = User.objects.first()
        user.is_superuser = False
        user.is_staff = False
        user.save()
        self.client.force_authenticate(user=user)
        response = self.client.get('/files/list-files/')
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)



import os
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

VERSION = '0.0.2'

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

REST = 'RST'
ORIGIN_CHOICES = (
    (REST, _('REST')),
)
DEFAULT = 'DFT'
TYPE_CHOICES = (
    (DEFAULT, _('Default')),
)
FILE_UPLOAD_EVENT = getattr(settings, 'FILE_UPLOAD_EVENT', 'FILE_UPLOAD_EVENT')

UPLOADS_DIR = getattr(settings, 'FILE_DIRS', 'file/')



from setuptools import setup, find_packages

setup(name='djangorestfilemanager',
      version='0.1.3',
      description='Django REST File Manager',
      url='https://gitlab.com/kas-factory/packages/django-rest-file-manager',
      author='Avelino @ KF',
      author_email='avelino@kasfactory.net',
      license='MIT',
      packages=find_packages(),
      package_data={
          'djangorestfilemanager': ['djangorestfilemanager/*',
                                    'djangorestfilemanager/migrations/*',
                                    'djangorestfilemanager/locale/en/LC_MESSAGES/django.po',
                                    'djangorestfilemanager/locale/es/LC_MESSAGES/django.po',
                                    'djangorestfilemanager/locale/en/LC_MESSAGES/django.mo',
                                    'djangorestfilemanager/locale/es/LC_MESSAGES/django.mo',
                                    'djangorestfilemanager/test/*'
                                    'djangorestfilemanager/test/files/*'
                                    ]
      },
      install_requires=[
          'Django',
          'djangorestframework',
          'django-storages',
          'django-filter',
          'django-cleanup',
          'djangopubsub',
          'boto3',
          'python-dateutil'
      ],
      zip_safe=False
      )



from django.contrib.auth.models import User
from django.test import override_settings
from rest_framework import status
from django.conf import settings
from rest_framework.test import APITestCase, APIClient

from djangorestfilemanager.models import File
from django.contrib.auth.models import Permission, ContentType


@override_settings(ROOT_URLCONF='files.urls')
class TestUploadFile(APITestCase):
    def setUp(self):
        self.content_type = ContentType.objects.create(app_label='file', model='file')
        self.perm = Permission.objects.create(content_type=self.content_type, codename='download_file',
                                              name='Download File')
        self.client = APIClient()
        self.user = User.objects.create(username='LUKE', password='1234qwer1234',
                                        email='luke@sw.stwars')
        self.user.user_permissions.add(self.perm)
        self.file = open('{}/djangorestfilemanager/test/files/test.png'.format(settings.BASE_DIR), 'rb')
        self.data = {
            'file': self.file,
            'name': 'test',
            'user_name': str(self.user.username)
        }
        self.client.force_authenticate(user=self.user)
        file = {'file': (str(self.file.name), self.file, 'text/plain', {'Expires': '0'})}
        self.response = self.client.post('/files/', data=self.data, file=file)
        self.assertEqual(self.response.status_code, status.HTTP_201_CREATED)

    def test_download_file_share(self):
        file = File.objects.first()
        file.share = True
        file.save()
        response = self.client.post('/files/{}/file/'.format(self.response.data.get('uuid')))
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_download_file(self):
        response = self.client.post('/files/{}/file/'.format(self.response.data.get('uuid')))
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_download_file_custom_permission_share(self):
        file = File.objects.first()
        file.share = True
        file.permission = '{}.{}'.format(self.perm.content_type, self.perm.codename)
        file.save()
        response = self.client.post('/files/{}/file/'.format(self.response.data.get('uuid')))
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_download_file_custom_permission(self):
        file = File.objects.first()
        file.permission = '{}.{}'.format(self.perm.content_type, self.perm.codename)
        file.save()
        response = self.client.post('/files/{}/file/'.format(self.response.data.get('uuid')))
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_download_file_wrong_custom_permission(self):
        file = File.objects.first()
        file.permission = 'wolas'
        file.save()
        response = self.client.post('/files/{}/file/'.format(self.response.data.get('uuid')))
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_download_file_wrong_custom_permission_share(self):
        file = File.objects.first()
        file.permission = 'wolas'
        file.share = True
        file.save()
        response = self.client.post('/files/{}/file/'.format(self.response.data.get('uuid')))
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_download_file_wrong_custom_permission_is_superuser(self):
        file = File.objects.first()
        file.permission = 'wolas'
        file.save()
        self.user.is_superuser = True
        self.user.save()
        response = self.client.post('/files/{}/file/'.format(self.response.data.get('uuid')))
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_download_file_wrong_user_name_share(self):
        file = File.objects.first()
        file.user_name = 'Chewaca'
        file.permission = 'wolas'
        file.save()
        response = self.client.post('/files/{}/file/'.format(self.response.data.get('uuid')))
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_download_file_not_authenticated(self):
        self.client.logout()
        response = self.client.post('/files/{}/file/'.format(self.response.data.get('uuid')))
        self.assertEqual(response.status_code, status.HTTP_401_UNAUTHORIZED)
